@@ -56,7 +56,7 @@ IrFunction *IRState::func() {
 }
 
 llvm::Function *IRState::topfunc() {
-  return func()->func;
+  return func()->getLLVMFunc();
 }
 
 llvm::Instruction *IRState::topallocapoint() {
@@ -126,6 +126,11 @@ LLCallSite IRState::CreateCallOrInvoke(LLValue *Callee, LLValue *Arg1,
   return funcGen().callOrInvoke(Callee, args, Name);
 }
 
+bool IRState::isMainFunc(const IrFunction *func) const {
+  assert(func != nullptr);
+  return func->getLLVMFunc() == mainFunc;
+}
+
 bool IRState::emitArrayBoundsChecks() {
   if (global.params.useArrayBounds != BOUNDSCHECKsafeonly) {
     return global.params.useArrayBounds == BOUNDSCHECKon;
