@@ -151,16 +151,22 @@ void DtoDefineNakedFunction(FuncDeclaration *fd) {
   // FIXME: could we perhaps use llvm asmwriter to give us these details ?
 
   const char *mangle = mangleExact(fd);
+  std::string fullmangle; // buffer only
+
   std::ostringstream tmpstr;
 
-  bool const isWin = global.params.targetTriple->isOSWindows();
-  bool const isOSX =
-      (global.params.targetTriple->getOS() == llvm::Triple::Darwin ||
-       global.params.targetTriple->getOS() == llvm::Triple::MacOSX);
+  const auto &triple = *global.params.targetTriple;
+  bool const isWin = triple.isOSWindows();
+  bool const isOSX = (triple.getOS() == llvm::Triple::Darwin ||
+                      triple.getOS() == llvm::Triple::MacOSX);
 
   // osx is different
   // also mangling has an extra underscore prefixed
   if (isOSX) {
+    fullmangle += '_';
+    fullmangle += mangle;
+    mangle = fullmangle.c_str();
+
     std::string section = "text";
     bool weak = false;
     if (DtoIsTemplateInstance(fd)) {
@@ -170,21 +176,29 @@ void DtoDefineNakedFunction(FuncDeclaration *fd) {
     }
     asmstr << "\t." << section << std::endl;
     asmstr << "\t.align\t4, 0x90" << std::endl;
-    asmstr << "\t.globl\t_" << mangle << std::endl;
+    asmstr << "\t.globl\t" << mangle << std::endl;
     if (weak) {
-      asmstr << "\t.weak_definition\t_" << mangle << std::endl;
+      asmstr << "\t.weak_definition\t" << mangle << std::endl;
     }
-    asmstr << "_" << mangle << ":" << std::endl;
+    asmstr << mangle << ":" << std::endl;
   }
   // Windows is different
   else if (isWin) {
-    std::string fullMangle;
-    if (!global.params.targetTriple->isArch64Bit()) {
-      fullMangle = "_";
+    // prepend extra underscore for Win32 (except for extern(C++))
+    if (triple.isArch32Bit() && fd->linkage != LINKcpp) {
+      fullmangle += '_';
+      fullmangle += mangle;
+      mangle = fullmangle.c_str();
+    }
+    // leading ? apparently needs quoting
+    else if (mangle[0] == '?') {
+      fullmangle += '"';
+      fullmangle += mangle;
+      fullmangle += '"';
+      mangle = fullmangle.c_str();
     }
-    fullMangle += mangle;
 
-    asmstr << "\t.def\t" << fullMangle << ";" << std::endl;
+    asmstr << "\t.def\t" << mangle << ";" << std::endl;
     // hard code these two numbers for now since gas ignores .scl and llvm
     // is defaulting to .type 32 for everything I have seen
     asmstr << "\t.scl 2;" << std::endl;
@@ -192,14 +206,14 @@ void DtoDefineNakedFunction(FuncDeclaration *fd) {
     asmstr << "\t.endef" << std::endl;
 
     if (DtoIsTemplateInstance(fd)) {
-      asmstr << "\t.section\t.text$" << fullMangle << ",\"xr\"" << std::endl;
+      asmstr << "\t.section\t.text$" << mangle << ",\"xr\"" << std::endl;
       asmstr << "\t.linkonce\tdiscard" << std::endl;
     } else {
       asmstr << "\t.text" << std::endl;
     }
-    asmstr << "\t.globl\t" << fullMangle << std::endl;
+    asmstr << "\t.globl\t" << mangle << std::endl;
     asmstr << "\t.align\t16, 0x90" << std::endl;
-    asmstr << fullMangle << ":" << std::endl;
+    asmstr << mangle << ":" << std::endl;
   } else {
     if (DtoIsTemplateInstance(fd)) {
       asmstr << "\t.section\t.text." << mangle << ",\"axG\",@progbits,"
