@@ -5,6 +5,8 @@
 // This file is distributed under the BSD-style LDC license. See the LICENSE
 // file for details.
 //
+// Implements some parts of the front-end Target class (ddmd/target.{d,h}).
+//
 //===----------------------------------------------------------------------===//
 
 #include "ldcbindings.h"
@@ -20,32 +22,90 @@
 #include <pthread.h>
 #endif
 
-/*
-These guys are allocated by ddmd/target.d:
-int Target::ptrsize;
-int Target::realsize;
-int Target::realpad;
-int Target::realalignsize;
-int Target::c_longsize;
-int Target::c_long_doublesize;
-bool Target::reverseCppOverloads;
-*/
+using llvm::APFloat;
+
+// target-real values
+real_t real_max;
+real_t real_min_normal;
+real_t real_epsilon;
+
+// Target::RealProperties functions
+real_t Target::RealProperties::nan() { return host_nan(); }
+real_t Target::RealProperties::snan() { return host_snan(); }
+real_t Target::RealProperties::infinity() { return host_infinity(); }
+real_t Target::RealProperties::max() { return real_max; }
+real_t Target::RealProperties::min_normal() { return real_min_normal; }
+real_t Target::RealProperties::epsilon() { return real_epsilon; }
 
 void Target::_init() {
   ptrsize = gDataLayout->getPointerSize(ADDRESS_SPACE);
 
-  llvm::Type *real = DtoType(Type::basic[Tfloat80]);
+  llvm::Type *const real = DtoType(Type::basic[Tfloat80]);
   realsize = gDataLayout->getTypeAllocSize(real);
   realpad = realsize - gDataLayout->getTypeStoreSize(real);
   realalignsize = gDataLayout->getABITypeAlignment(real);
-  c_longsize = global.params.is64bit ? 8 : 4;
-  c_long_doublesize = realsize;
+  realislongdouble = true;
 
   // according to DMD, only for MSVC++:
   reverseCppOverloads = global.params.targetTriple->isWindowsMSVCEnvironment();
 
   // LDC_FIXME: Set once we support it.
   cppExceptions = false;
+
+  c_longsize = global.params.is64bit ? 8 : 4;
+  c_long_doublesize = realsize;
+  classinfosize = 0; // unused
+
+  const auto targetRealSemantics = &real->getFltSemantics();
+#if LDC_LLVM_VER >= 400
+  const auto IEEEdouble = &APFloat::IEEEdouble();
+  const auto x87DoubleExtended = &APFloat::x87DoubleExtended();
+  const auto IEEEquad = &APFloat::IEEEquad();
+#else
+  const auto IEEEdouble = &APFloat::IEEEdouble;
+  const auto x87DoubleExtended = &APFloat::x87DoubleExtended;
+  const auto IEEEquad = &APFloat::IEEEquad;
+#endif
+
+  if (targetRealSemantics == IEEEdouble) {
+    real_max = CTFloat::parse("0x1.fffffffffffffp+1023");
+    real_min_normal = CTFloat::parse("0x1p-1022");
+    real_epsilon = CTFloat::parse("0x1p-52");
+    RealProperties::dig = 15;
+    RealProperties::mant_dig = 53;
+    RealProperties::max_exp = 1024;
+    RealProperties::min_exp = -1021;
+    RealProperties::max_10_exp = 308;
+    RealProperties::min_10_exp = -307;
+  } else if (targetRealSemantics == x87DoubleExtended) {
+    real_max = CTFloat::parse("0x1.fffffffffffffffep+16383");
+    real_min_normal = CTFloat::parse("0x1p-16382");
+    real_epsilon = CTFloat::parse("0x1p-63");
+    RealProperties::dig = 18;
+    RealProperties::mant_dig = 64;
+    RealProperties::max_exp = 16384;
+    RealProperties::min_exp = -16381;
+    RealProperties::max_10_exp = 4932;
+    RealProperties::min_10_exp = -4931;
+  } else if (targetRealSemantics == IEEEquad) {
+    // FIXME: hex constants
+    real_max = CTFloat::parse("1.18973149535723176508575932662800702e+4932");
+    real_min_normal =
+        CTFloat::parse("3.36210314311209350626267781732175260e-4932");
+    real_epsilon = CTFloat::parse("1.92592994438723585305597794258492732e-34");
+    RealProperties::dig = 33;
+    RealProperties::mant_dig = 113;
+    RealProperties::max_exp = 16384;
+    RealProperties::min_exp = -16381;
+    RealProperties::max_10_exp = 4932;
+    RealProperties::min_10_exp = -4931;
+  } else {
+    // rely on host compiler
+    real_max = RealProperties::host_max();
+    real_min_normal = RealProperties::host_min_normal();
+    real_epsilon = RealProperties::host_epsilon();
+    // the rest is already initialized with the corresponding real_t values
+  }
 }
 
 /******************************
@@ -118,6 +178,18 @@ unsigned Target::critsecsize() {
 
 Type *Target::va_listType() { return gABI->vaListType(); }
 
+/******************************
+ * Check if the given type is supported for this target
+ * 0: supported
+ * 1: not supported
+ * 2: wrong size
+ * 3: wrong base type
+ */
+int Target::checkVectorType(int sz, Type *type) {
+  // FIXME: Is it possible to query the LLVM target about supported vectors?
+  return 0;
+}
+
 /******************************
  * Encode the given expression, which is assumed to be an rvalue literal
  * as another type for use in CTFE.
@@ -166,10 +238,10 @@ Expression *Target::paintAsType(Expression *e, Type *type) {
     return createIntegerExp(e->loc, u.int64value, type);
 
   case Tfloat32:
-    return createRealExp(e->loc, ldouble(u.float32value), type);
+    return createRealExp(e->loc, u.float32value, type);
 
   case Tfloat64:
-    return createRealExp(e->loc, ldouble(u.float64value), type);
+    return createRealExp(e->loc, u.float64value, type);
 
   default:
     assert(0);
@@ -178,18 +250,6 @@ Expression *Target::paintAsType(Expression *e, Type *type) {
   return nullptr; // avoid warning
 }
 
-/******************************
- * Check if the given type is supported for this target
- * 0: supported
- * 1: not supported
- * 2: wrong size
- * 3: wrong base type
- */
-int Target::checkVectorType(int sz, Type *type) {
-  // FIXME: It is possible to query the LLVM target about supported vectors?
-  return 0;
-}
-
 /******************************
  * For the given module, perform any post parsing analysis.
  * Certain compiler backends (ie: GDC) have special placeholder
@@ -202,4 +262,3 @@ void Target::loadModule(Module *m) {}
  *
  */
 void Target::prefixName(OutBuffer *buf, LINK linkage) {}
-
