@@ -102,7 +102,7 @@ LLConstant *IrAggr::getDefaultInitializer(VarDeclaration *field) {
   if (field->_init) {
     // Issue 9057 workaround caused by issue 14666 fix, see DMD upstream
     // commit 069f570005.
-    if (field->sem < Semantic2Done && field->_scope) {
+    if (field->semanticRun < PASSsemantic2done && field->_scope) {
       field->semantic2(field->_scope);
     }
     return DtoConstInitializer(field->_init->loc, field->type, field->_init);
